# -*- coding: utf-8 -*-

"""
@date: 2020/7/14 下午8:34
@file: anno_processor.py
@author: zj
@description: 
"""

import os
import glob

from parseanno.anno import build_anno
from parseanno.utils.logger import setup_logger


class AnnoProcessor(object):
    """
    对标注数据进行处理，创建指定格式的训练数据
    """

    def __init__(self, cfg):
        self.parser = build_anno(cfg.ANNO.PARSER, cfg)
        self.creator = build_anno(cfg.ANNO.CREATOR, cfg)

        self.logger = setup_logger(__name__)

    def process(self):
        self.logger.info('处理原始标注数据')
        anno_data = self.parser.process()
        self.logger.info('保存指定格式数据')
        self.creator.save(anno_data)
        self.logger.info('完成')



# -*- coding: utf-8 -*-

"""
@date: 2020/7/21 下午8:32
@file: parse-anno.py
@author: zj
@description: 
"""

from parseanno.config import cfg
from parseanno.engine import AnnoProcessor, default_argument_parser


def main(parser):
    args = parser.parse_args()

    cfg.merge_from_file(args.config_file)
    cfg.freeze()

    processor = AnnoProcessor(cfg)
    processor.process()


if __name__ == '__main__':
    main(default_argument_parser())



# -*- coding: utf-8 -*-

"""
@date: 2020/7/17 下午7:29
@file: tlt_anno.py
@author: zj
@description: 
"""

import os
import cv2
import numpy as np
import json

from parseanno.anno.base_anno import BaseAnno
from parseanno.utils.utility import get_file_name
from parseanno.anno import registry

from parseanno.utils.logger import setup_logger


@registry.ANNOS.register('tlt')
class TltAnno(BaseAnno):
    """
    [Nvidia TLT](https://docs.nvidia.com/metropolis/TLT/tlt-getting-started-guide/index.html)训练需要Kitti格式
    参考：
    [Object Detection Data Extension](https://github.com/NVIDIA/DIGITS/blob/v4.0.0-rc.3/digits/extensions/data/objectDetection/README.md)
    [Object Detection: About KITTI format #992](https://github.com/NVIDIA/DIGITS/issues/992)
    每行表示一个标注对象，共15个字段（空格隔开），第一个字段表示类名，第5-9表示xmin/ymin/xmax/ymax（浮点表示），其余设为0即可
    整体保存架构
    root/
        images/
            1.png
            2.png
        labels/
            1.txt
            2.txt
    """

    def __init__(self, cfg):
        self.dst_dir = cfg.TLT.DST_DIR
        self.img_extension = cfg.TLT.IMG_EXTENSION
        self.anno_extension = cfg.TLT.ANNO_EXTENSION
        self.save_classmap = cfg.ANNO.SAVE_CLASSMAP
        self.verbose = cfg.ANNO.VERBOSE

        self.logger = setup_logger(__name__)

    def process(self) -> dict:
        pass

    def save(self, anno_data):
        super(TltAnno, self).save(anno_data)

        dst_dir = self.dst_dir
        save_classmap = self.save_classmap
        verbose = self.verbose

        classmap = anno_data['classmap']

        dst_img_dir = os.path.join(dst_dir, 'images')
        dst_label_dir = os.path.join(dst_dir, 'labels')
        if not os.path.exists(dst_dir):
            os.mkdir(dst_dir)
        if os.path.exists(dst_img_dir):
            raise ValueError('{}已存在'.format(dst_img_dir))
        if os.path.exists(dst_label_dir):
            raise ValueError('{}已存在'.format(dst_label_dir))
        os.mkdir(dst_img_dir)
        os.mkdir(dst_label_dir)

        for i, (img_path, anno_obj) in enumerate(anno_data['anno_data'].items(), 1):
            img_name = get_file_name(img_path)
            if verbose:
                self.logger.info('保存{}'.format(img_name))

            size = anno_obj['size']
            objects = anno_obj['objects']

            label_list = list()
            for obj in objects:
                name = obj['name']
                bndbox = obj['bndbox']

                xmin, ymin, xmax, ymax = bndbox
                label_list.append([name, 0, 0, 0, xmin, ymin, xmax, ymax, 0, 0, 0, 0, 0, 0, 0])
            # 保存
            img = cv2.imread(img_path)
            dst_img_path = os.path.join(dst_img_dir, img_name + self.img_extension)
            cv2.imwrite(dst_img_path, img)

            dst_label_path = os.path.join(dst_label_dir, img_name + self.anno_extension)
            np.savetxt(dst_label_path, label_list, fmt='%s', delimiter=' ')
        if save_classmap:
            self.logger.info('保存classmap')
            classmap_path = os.path.join(dst_dir, 'classmap.json')
            with open(classmap_path, 'w') as f:
                json.dump(classmap, f)
        if verbose:
            print(__name__, 'done')



# -*- coding: utf-8 -*-

"""
@date: 2020/7/14 下午7:39
@file: yolov5_anno.py
@author: zj
@description: 
"""

import os
import copy
import shutil
import cv2
import numpy as np
import json

from parseanno.utils.utility import get_file_name
from parseanno.anno import registry
from parseanno.anno.base_anno import BaseAnno
from parseanno.utils.logger import setup_logger


@registry.ANNOS.register('yolov5')
class YoLoV5Anno(BaseAnno):
    """
    创建[ultralytics/yolov5](https://github.com/ultralytics/yolov5)训练所需数据集
    其图像和标注文件排列如下：
    ├── ocr-task3-detect
       ├── images
            ├── 201.png
            ├── 202.png
       └── labels
            ├── 201.txt
            ├── 202.txt
    标注文件格式如下：
    class_num x_center y_center width height
    class_num表示类别，从0开始
    x_center/y_center/width/height使用相对图像宽高的坐标
    示例如下：
    0 0.110547 0.051621 0.154554 0.023163

    额外保存一个classmap.txt，保存class_num和对应类名
    """

    def __init__(self, cfg):
        self.dst_dir = cfg.YOLOV5.DST_DIR
        self.img_extension = cfg.YOLOV5.IMG_EXTENSION
        self.anno_extension = cfg.YOLOV5.ANNO_EXTENSION
        self.save_classmap = cfg.ANNO.SAVE_CLASSMAP
        self.verbose = cfg.ANNO.VERBOSE

        self.logger = setup_logger(__name__)

    def xyxy_2_xywh(self, bndbox, size):
        """
        创建yolov5
        :param bndbox: [xmin, ymin, xmax, ymax]
        :param size: (width, height)
        :return: [x_center, y_center, width, height]相对格式
        """
        xmin, ymin, xmax, ymax = bndbox
        img_w, img_h = size[:2]

        x_center = (xmin + xmax) / 2
        y_center = (ymin + ymax) / 2
        width = xmax - xmin
        height = ymax - ymin

        x_center = 1.0 * x_center / img_w
        y_center = 1.0 * y_center / img_h
        width = 1.0 * width / img_w
        height = 1.0 * height / img_h

        return [x_center, y_center, width, height]

    def process(self) -> dict:
        pass

    def save(self, anno_data):
        super(YoLoV5Anno, self).save(anno_data)

        dst_dir = self.dst_dir
        save_classmap = self.save_classmap
        verbose = self.verbose

        classmap = anno_data['classmap']

        dst_img_dir = os.path.join(dst_dir, 'images')
        dst_label_dir = os.path.join(dst_dir, 'labels')
        if not os.path.exists(dst_dir):
            os.mkdir(dst_dir)
        if os.path.exists(dst_img_dir):
            raise ValueError('{}已存在'.format(dst_img_dir))
        if os.path.exists(dst_label_dir):
            raise ValueError('{}已存在'.format(dst_label_dir))
        os.mkdir(dst_img_dir)
        os.mkdir(dst_label_dir)

        for i, (img_path, anno_obj) in enumerate(anno_data['anno_data'].items(), 1):
            img_name = get_file_name(img_path)
            if verbose:
                self.logger.info('保存{}'.format(img_name))

            size = anno_obj['size']
            objects = anno_obj['objects']

            label_list = list()
            for obj in objects:
                name = obj['name']
                bndbox = obj['bndbox']

                x_center, y_center, box_w, box_h = self.xyxy_2_xywh(bndbox, size)
                label_list.append([classmap[name], x_center, y_center, box_w, box_h])
            # 保存
            img = cv2.imread(img_path)
            dst_img_path = os.path.join(dst_img_dir, img_name + self.img_extension)
            cv2.imwrite(dst_img_path, img)

            dst_label_path = os.path.join(dst_label_dir, img_name + self.anno_extension)
            np.savetxt(dst_label_path, label_list, fmt='%d %.6f %.6f %.6f %.6f', delimiter=' ')
        if save_classmap:
            classmap_path = os.path.join(dst_dir, 'classmap.json')
            with open(classmap_path, 'w') as f:
                json.dump(classmap, f)
        if verbose:
            self.logger.info(__name__, 'done')



# -*- coding: utf-8 -*-

"""
@date: 2020/7/13 下午4:11
@file: utility.py
@author: zj
@description: 常用操作
"""

import cv2
import os
import glob


def is_dir(src_dir):
    """
    判断是否是目录
    """
    if not os.path.isdir(src_dir):
        raise ValueError('{}不是目录'.format(src_dir))


def get_file_name(file_path):
    """
    解析文件名
    """
    return os.path.splitext(os.path.split(file_path)[1])[0]


def get_classmap(anno_data, classmap):
    """
    从中间格式标注数据中找到classmap
    :param anno_list: [{'size':(width, height), 'objects':[{'name': '', 'bndbox':[xmin,ymin,xmax,ymax]}, ...]}, ...]
    :param classmap: 已有类名和数字对应图
    :return: {'a': 0, 'b':1 ,...}
    """
    for _, anno_obj in anno_data.items():
        objects = anno_obj['objects']
        for obj in objects:
            name = obj['name']
            if name not in classmap.keys():
                classmap[name] = len(classmap.keys())
    return classmap


def check(img_path_list, anno_path_list):
    """
    检查图像文件和标注文件是否一一对应
    """
    assert len(img_path_list) == len(anno_path_list), \
        '图像文件数：{} - 标注文件数：{}'.format(len(img_path_list), len(anno_path_list))

    for img_path, anno_path in zip(img_path_list, anno_path_list):
        img_name = get_file_name(img_path)
        anno_name = get_file_name(anno_path)

        if img_name != anno_name:
            raise ValueError('{}和{}不对应'.format(img_path, anno_path))



# -*- coding: utf-8 -*-

"""
@date: 2020/7/21 下午9:27
@file: setup.py.py
@author: zj
@description: 
"""

import setuptools
from os import path


def get_version():
    init_py_path = path.join(path.abspath(path.dirname(__file__)), "parseanno", "__init__.py")
    init_py = open(init_py_path, "r").readlines()
    version_line = [l.strip() for l in init_py if l.startswith("__version__")][0]
    version = version_line.split("=")[-1].strip().strip("'\"")

    return version


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ParseAnno",
    version=get_version(),
    author="zj",
    author_email="wy163zhuj@163.com",
    description="Script for annotation data processing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zjZSTU/ParseAnno",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: Apache Software License"
    ],
    python_requires='>=3.6',
    install_requires=[
        "yacs >= 0.1.7",
        "opencv_contrib_python >= 4.2.0",
        "numpy >= 1.17.2"
    ]
)
